/*
 * SDL_Animation son unas librerías para crear animacion en SDL
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Estas librerías tienen licencia Creative Commons y se permite
 * su modificación y utilización libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas información visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cádiz (Spain)
 * javi.pacheco@terra.es
 *
 */
 
#ifndef SDL_ANIMATION_H_
#define SDL_ANIMATION_H_

static char archivoComprimido[27];

 // tipo coordenadas
#define COORDENADASNORMALES 0
#define COORDENADASPOLARES 1

 // alpha
#define IMAGENSINALPHA 0
#define IMAGENCONALPHA 1

 // tipos de rotaciones
#define ROTACION_PENDULAR 0
#define ROTACION_CIRCULAR 1

// alineacion de fondos

#define BG_TOP_LEFT 0
#define BG_TOP_RIGHT 1
#define BG_BOTTOM_LEFT 2
#define BG_BOTTOM_RIGHT 3
#define BG_CENTER 4
#define BG_MOSAIC 5

// tipo de trabajo 
#define DESARROLLO 0
#define EXPLOTACION 1

// alineaciones
#define ALIGN_LEFT 0 
#define ALIGN_CENTER 1 
#define ALIGN_RIGHT 2 
#define VALIGN_TOP 0 
#define VALIGN_CENTER 1 
#define VALIGN_BOTTOM 2 

#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <SDL_mixer.h>
#include "SDL_rotozoom.h"
#include "unrarlib.h"
#include "defines.h"
#include <string.h>
#include <expat.h>
#include <map>
#include <string>
#include <cmath>
#include <iostream>
#include <vector>

using namespace std;

class SDL_Animation;
class Objeto;
class ObjetoForeground;
	
typedef struct
{
	int cicloini, ciclofin, posxini, posyini, posxfin, posyfin;
	int tiempo_fadein, tiempo_fadeout;
	int fadein_factor_begin, fadein_factor_end, fadeout_factor_begin, fadeout_factor_end;
	int tiempo_cortinillaH_in, tiempo_cortinillaH_out, tiempo_cortinillaV_in, tiempo_cortinillaV_out;
	bool curtainVInUpDown;
	bool curtainHInLeftRight;
	bool curtainVOutUpDown;
	bool curtainHOutLeftRight;
	int tiempo_zoom_in;
	double tiempo_zoom_in_factor_begin, tiempo_zoom_in_factor_end;
	int tiempo_zoom_out;
	double tiempo_zoom_out_factor_begin, tiempo_zoom_out_factor_end;
	int tiempo_rotate_in, tiempo_rotate_out;
	int factorRotatein, factorRotateout;
	int rotatein_angle_begin, rotatein_angle_end;
	int rotateout_angle_begin, rotateout_angle_end;
} Movimiento;

typedef struct
{
	int isPlay; // 1 para play y 0 para stop
	int cycle;
	int frame;
} ControlAnimation;

typedef struct
{
	int cycle, red, green, blue, position;
	Objeto *fondo;
} Fondo;

typedef struct
{
	int cicloIni, cicloFin, tiempoFadeIn, tiempoFadeOut, loops;
} Esquema;

typedef struct
{
	int cycle_begin, cycle_end;
	int tiempoFadeOut;
	int tiempo_cortinillaH_out, tiempo_cortinillaV_out;
	int tiempo_zoom_out;
	double tiempo_zoom_out_factor_begin, tiempo_zoom_out_factor_end;
	ObjetoForeground *fg;
} Foreground;

void parseXML(SDL_Animation *anim, char archXML[64]);
void parseBuffer(SDL_Animation *anim, string buffer);
 
class FrameObjeto {
	
public:
	SDL_Surface *img;
	SDL_RWops* RWFromRar(char *fileRar, char *file);
	void load(char *path);
	void load(char *path, SDL_Surface *sc);
	void loadSurface (SDL_Surface *surface);
	void loadAlpha(char *path);
	void unLoad();

};

class Objeto {

private:
	float posX, posY, oldX, oldY;
	vector<Movimiento> movimientos;
	vector<ControlAnimation> controlAnimation;
	bool isAnimacionParada;
	int numMovimientos;
	int dx, dy; // dirección x e y en el caso que sea necesario
	float polarCoseno, polarSeno, polarDistancia; // direccion con coordenadas polares
	int tipoCoordenadas; // 0 -> usa cordenadas normales 1-> usa coordenadas polares
	char nombre[27]; // nombre del elemento
	int actualFrame;
	int nFrames;
	int cont;
	int activo; // comprueba si el elemento está activo en la pantalla, caso de las balas por ejemplo
					// que se crean con arrays
	int fantasma;
	Uint32 *transparente; // valor transparente de la imagen
	//int impactosParaMorir;
	//int actualImpactos;
	int tipoElemento;
	int rebote; // para la funcion selSiguienteFrame_Rebote
	int ciclo;
	int movimientoActual;
	int cicloNacimiento, cicloMuerte;
	int ciclosEntreFrames; // para animaciones
	int tipoRotacion; // para animaciones: pendular o circular
	// variables para fade
	int alpha; // factor alpha aplicado al elemento
	int tiempoFadeIn, tiempoFadeOut, tiempoFadeInTotal, tiempoFadeOutTotal;
	int fadein_factor_begin, fadein_factor_end;
	int fadeout_factor_begin, fadeout_factor_end;
	// variables para cortinillas
	int cortinillaVIn, maxCortinillaVIn;
	int cortinillaHIn, maxCortinillaHIn;
	int cortinillaVOut, maxCortinillaVOut;
	int cortinillaHOut, maxCortinillaHOut;
	bool curtainVInUpDown;
	bool curtainHInLeftRight;
	bool curtainVOutUpDown;
	bool curtainHOutLeftRight;
	// variables para zoom
	int zoomin, zoomout;
	int maxZoomin, maxZoomout;
	double zoomin_factor_begin, zoomin_factor_end;
	double zoomout_factor_begin, zoomout_factor_end;
	double factor_zoom_actual; // si es 1, no hará ZOOM
	// variables para rotaciones
	int rotatein, rotateout;
	int maxRotatein, maxRotateout;
	int factorRotatein, factorRotateout; // 1 (SCAR) y -1 (SAR)
	int rotatein_angle_begin, rotatein_angle_end;
	int rotateout_angle_begin, rotateout_angle_end;
	int angle_rotate_actual;
	// variables para alineaciones
	int align, valign;
	int bgR, bgG, bgB; // colores para utilizar de fondo en el caso de tener un alpha la imagen
	bool imagen_conAlpha; // nos dice si la imagen PNG contiene alpha
	
public:
	FrameObjeto *sprite;
	Objeto ();
	Objeto (int nf);
	Objeto (int nf, int ni);
	void unLoad ();
	void addFrame (FrameObjeto frame);
	void modificarFrameN (FrameObjeto frame, int n);
	void selFrame (int nf);
	int getFrames () { return nFrames; }
	void selSiguienteFrame();
	void selSiguienteFrame_Rebote();
	void selSiguienteFrame_ReboteHasta(int hasta);
	void setNombre (char n[15]) { strcpy(nombre, n); }
	char* getNombre () { return nombre; }
	void playAnimacion () { isAnimacionParada = 0; }
	void stopAnimacion () { isAnimacionParada = 1; }
	void setCiclo(int c) { ciclo = c; }
	int getCiclo() { return ciclo; }
	void addCiclo() { ciclo++; }
	void setColoresFondoEnAlpha(int r, int g, int b) { bgR = r; bgG = g; bgB = b; };
	void setImagen_ConAlpha(bool a) { imagen_conAlpha = a; };
	bool getImagen_ConAlpha() { return imagen_conAlpha; };
	void setMovimiento (int ciclo, int x, int y, int xFin, int yFin, int tiempo, int desaparecerAlFinal, int tiempoFadeIn, int fadein_opacidad_begin, int fadein_opacidad_end, int tiempoFadeOut, int fadeout_opacidad_begin, int fadeout_opacidad_end, int tiempoCortinillaHIn, int tiempoCortinillaHOut, int tiempoCortinillaVIn, int tiempoCortinillaVOut);
	void setFadeIn(int t, int begin, int end) { alpha = begin; fadein_factor_begin = begin; fadein_factor_end = end; tiempoFadeIn = t; tiempoFadeInTotal = t; }
	int getFadeIn() { return tiempoFadeIn; }
	void setFadeOut(int t, int begin, int end) { alpha = begin; fadeout_factor_begin = begin; fadeout_factor_end = end; tiempoFadeOut = t; tiempoFadeOutTotal = t; }
	int getFadeOut() { return tiempoFadeOut; }
	void setCortinillaVIn_EnMovimientoActual(int tiempoCortinillaVIn, bool UpDown);
	void setCortinillaHIn_EnMovimientoActual(int tiempoCortinillaHIn, bool LeftRight);
	void setCortinillaVOut_EnMovimientoActual(int tiempoCortinillaVOut, bool UpDown);
	void setCortinillaHOut_EnMovimientoActual(int tiempoCortinillaHOut, bool LeftRight);
	void setZoomIn_EnMovimientoActual(int tiempoZoomIn, double factor_begin, double factor_end);
	void setZoomOut_EnMovimientoActual(int tiempoZoomOut, double factor_begin, double factor_end);
	void setRotateIn_EnMovimientoActual(int tiempoRotateIn, int angle_begin, int angle_end, int factor);
	void setRotateOut_EnMovimientoActual(int tiempoRotateOut, int angle_begin, int angle_end, int factor);
	void setCicloNacimiento (int c) { cicloNacimiento=c; }
	void setCicloMuerte (int c) { cicloMuerte=c; }
	int getCicloNacimiento () { return cicloNacimiento; }
	int getCicloMuerte () { return cicloMuerte; }
	void setCiclosEntreFrames (int c) { ciclosEntreFrames = c; }
	int getCiclosEntreFrames() { return ciclosEntreFrames; }
	void setTipoRotacion (int r) { tipoRotacion = r; }
	int getTipoRotacion() { return tipoRotacion; }
	void setCortinillaHIn (int c, bool LeftRight) { curtainHInLeftRight = LeftRight; maxCortinillaHIn = c; cortinillaHIn = c; }
	void setCortinillaHIn (int c) { maxCortinillaHIn = c; cortinillaHIn = c; }
	int getCortinillaHIn() { return cortinillaHIn; }
	void setCortinillaVIn (int c, bool UpDown) { curtainVInUpDown = UpDown; maxCortinillaVIn = c; cortinillaVIn = c; }
	void setCortinillaVIn (int c) { maxCortinillaVIn = c; cortinillaVIn = c; }
	int getCortinillaVIn() { return cortinillaVIn; }
	void setCortinillaHOut (int c, bool LeftRight) { curtainHOutLeftRight = LeftRight; maxCortinillaHOut = c; cortinillaHOut = c; }
	void setCortinillaHOut (int c) { maxCortinillaHOut = c; cortinillaHOut = c; }
	int getCortinillaHOut() { return cortinillaHOut; }
	void setCortinillaVOut (int c, bool UpDown) { curtainVOutUpDown = UpDown; maxCortinillaVOut = c; cortinillaVOut = c; }
	void setCortinillaVOut (int c) { maxCortinillaVOut = c; cortinillaVOut = c; }
	int getCortinillaVOut() { return cortinillaVOut; }
	void setZoom(double z) { factor_zoom_actual = z; }
	void setZoomIn(int z) { maxZoomin = z; zoomin = z; }
	int getZoomIn() { return zoomin; }
	void setZoomOut(int z) { maxZoomout = z; zoomout = z; }
	int getZoomOut() { return zoomout; }
	void setZoomInFactorBegin(double f) { zoomin_factor_begin = f; }
	double getZoomInFactorBegin() { return zoomin_factor_begin; }
	void setZoomInFactorEnd(double f) { zoomin_factor_end = f; }
	double getZoomInFactorEnd() { return zoomin_factor_end; }
	void setZoomOutFactorBegin(double f) { zoomout_factor_begin = f; }
	double getZoomOutFactorBegin() { return zoomout_factor_begin; }
	void setZoomOutFactorEnd(double f) { zoomout_factor_end = f; }
	double getZoomOutFactorEnd() { return zoomin_factor_end; }
	void setRotate(int r) { angle_rotate_actual = r; }
	void setRotateIn(int r) { maxRotatein = r; rotatein = r; }
	int getRotateIn() { return rotatein; }
	void setRotateOut(int r) { maxRotateout = r; rotateout = r; }
	int getRotateOut() { return rotateout; }
	void setRotateInAngles (int begin, int end, int factor) { rotatein_angle_begin = begin; rotatein_angle_end = end; if (begin < end) { factorRotatein=-factor; } else { factorRotatein=factor; } }
	void setRotateOutAngles (int begin, int end, int factor) { rotateout_angle_begin = begin; rotateout_angle_end = end; if (begin < end) { factorRotateout=-factor; } else { factorRotateout=factor; } }
	void addControlAnimation(ControlAnimation ca) { controlAnimation.push_back(ca); };
	// alineaciones
	void setAlign (int a) { align = a; }
	int getAlign() { return align; }
	void setVAlign (int a) { valign = a; }
	int getVAlign() { return valign; }
	// límites del elemento en la pantalla
	// se utilizan para saber cuando un elemento tiene que ser descargado al salir de la pantalla
	int getLimiteSup () { return (-getAlto() * 2) -5; }
	int getLimiteInf () { return 600 + (getAlto() * 2) -5; }
	int getLimiteDer () { return 650 + getAncho() * 2 -5; }
	int getLimiteIzq () { return -getAncho() * 2 -5; }
	bool fueraLimitesDePantalla ();
	float getCentroX () { return posX + getAncho()/2; }
	float getCentroY () { return posY + getAlto()/2; }
	int getCentrarXen (int x) { return x - getAncho()/2; }
	int getCentrarYen (int y) { return y - getAlto()/2; }
	void setTipoCoordenadas (int c) { tipoCoordenadas = c; }
	int getTipoCoordenadas () { return tipoCoordenadas; }
	void setDPolarCoseno (float c) { polarCoseno = c; }
	void setDPolarSeno (float s) { polarSeno = s; }
	void setDPolarDistancia (float d) { polarDistancia = d; }
	float getDPolarCoseno () { return polarCoseno; }
	float getDPolarSeno () { return polarSeno; }
	float getDPolarDistancia () { return polarDistancia; }
	void setDX (int x) { dx = x; }
	void setDY (int y) { dy = y; }
	int getDX () { return dx; }
	int getDY () { return dy; }
	void addDX () { posX += dx; }
	void addDY () { posY += dy; }
	void setX (int x) { oldX = posX; posX = x; }
	void setY (int y) { oldY = posY; posY = y; }
	void addX (int x) { posX += x; }
	void addY (int y) { posY += y; }
	float getOldX () { return posX; }
	float getOldY () { return posY; }
	float getX () { return posX; }
	float getY () { return posY; }
	void mover ();
	void comprobarMovimiento ();
	int getAncho () { return sprite[actualFrame].img->w; }
	int getAlto () { return sprite[actualFrame].img->h; }
	void setActivo (int a) { activo = a; }
	int getActivo () { return activo; }
	int getActualFrame () { return actualFrame; }
	void setFantasma (int nf) { fantasma = nf; };
	void conmutaFantasma ();
	int getFantasma () { return fantasma; };
	void setAlpha (int a) { alpha = a; };
	void removeAlpha () { alpha = -1; };
	void dibujar(SDL_Surface *superficie);
	SDL_Surface* getSurface() { return sprite[actualFrame].img;};
	Uint32 SDL_GetPixel(SDL_Surface *surface, int x, int y);
};


class ObjetoForeground : public Objeto {
	
private:
	int colorFondo;
	int ancho, alto;

public:
	ObjetoForeground (SDL_Surface *screen, int anc, int alt);
	void crearForeground (SDL_Surface *screen, int r, int g, int b, char imagen[127], int posicionFondo, int conAlpha, bool fondoTransparente);
};

class Sonido {
	
private:
	int ciclo;
	int numEsquemas;
	int activo;
	int canalAudio;
	char nombre[27]; // nombre del sonido
	//int esquema[NUMSONIDOS][5]; // 0 cicloini, 1 ciclofin, 2 tiempoFadeIn, 3 tiempoFadeOut, 4 loops
	vector<Esquema> esquema;
	Esquema auxEsquema;

public:
	Mix_Chunk *musica;
	Sonido(char archivo[127], int volumen);
	SDL_RWops* RWFromRar(char *fileRar, char *file);
	void setNombre (char n[27]) { strcpy(nombre, n); }
	void setCanalAudio(int c) { canalAudio = c; }
	bool play(int i);
	bool stop(int i);
	char* getNombre () { return nombre; };
	void setEsquema(int cicloIni, int cicloFin, int tiempoFadeIn, int tiempoFadeOut, int loops);
	void activar();
	void unLoad ();
};

class SDL_Animation {
	
private:
	vector<Objeto*> elementos;
	int numeroElementos;
	Objeto *loading;
	int ancho, alto;
	int tipoTrabajo;
	int posicionNombreObjeto (char name[27]);
	int posicionNombreSonido (char name[27]);
	Objeto *fondoDesarrollo;
	Uint32 colorFondo;
	SDL_Rect rectFondo;
	int ciclo;
	int numeroSonidos;
	vector<Sonido*> sonidos;
	vector<Fondo> fondos;
	int numeroFondos;
	int imagenFondoActual; // numero del array que contiene el fondo a mostrar segun el ciclo
	map<string, string> variables;
	int cicloFin; // ciclo en que la aplicación termina. Devuelve 1 el metodo escena
	vector<Foreground> fg;
	int numeroForeground;
	char fuenteTTFParaDevelop[127];
	bool mostrarCartel; // en el caso que esté en trabajo desarrollo la tecla "c"
						// cambia esta variable para mostrar u ocultar el cartel
	bool pausa;
	bool forzarFinAnimacion; // sale de la animacion
	int empezarEnCiclo;
	SDL_Event event;
	bool salirConTodasLasTeclas;
	
public:
	SDL_Surface *screen; // pantalla de la animacion
	vector<int> stop;
	Objeto *textStop;

	SDL_Animation (SDL_Surface *srf_screen, int anc, int alt);
	void loadXML(char archXML[64], char archRAR[27]);
	void loadBuffer (string b, char archRAR[27]);
	int getAncho() { return ancho; }
	int getNumElementos() { return numeroElementos; }
	int getNumSonidos() { return numeroSonidos; }
	int getAlto() { return alto; }
	void setSalirConTodasLasTeclas(bool s) { salirConTodasLasTeclas = s; }
	bool getSalirConTodasLasTeclas() { return salirConTodasLasTeclas; }
	void setTrabajo(int t) { tipoTrabajo = t; }
	void setCicloFin(int c) { cicloFin = c; }
	void setFuenteTTFParaDevelop(char f[127]) { strcpy(fuenteTTFParaDevelop, f); }
	int getCicloFin() { return cicloFin; }
	void setCiclo(int c) { ciclo = c; }
	void comprobarTeclas ();
	void imprimirValoresObjeto (char nombre[15]);
	void imprimirValoresTodosLosObjeto ();
	void setEmpezarEnCiclo(int c) { empezarEnCiclo = c; }
	void cambiarCicloAnimacion (int c);
	void crearLoading(char imagen[127], int conAlpha);
	// control de objetos
	void crearElemento(char nombre[15], char imagen[127], int conAlpha);
	void crearElementoAnimacion(char nombre[15], int numElementos, int ciclosEntreFrames, int rotacion);
	void addImagenElementoAnimacion (char nombre[15], char imagen[127], int conAlpha);
	void crearElementoAnimacion(char nombre[15], int numElementos, int ciclosEntreFrames, int rotacion, char imagen[127], char ext[5], int conAlpha);
	SDL_Surface* surfaceLinea (char fuenteTTF[127], int tamTexto, char txt[127], int fondoTranparente, int fgR, int fgG, int fgB, int bgR, int bgG, int bgB);
	void crearTexto(char nombre[15], char texto[127], int tamTexto, char fuenteTTF[127], int text_align, int fondoTranparente, int separacionInterlineal, int x, int y, int fgR, int fgG, int fgB, int bgR, int bgG, int bgB);
	void crearRectangulo(char nombre[15], SDL_Color color, int anchura, int altura);
	void addMovimiento (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal);
	void addPosicion (char nombre[15], int cicloIni, int posX, int posY);
	void addMovimiento_FadeIn (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal, int tiempoFadeIn, int opacidad_begin, int opacidad_end);
	void addMovimiento_FadeOut (char name[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal, int tiempoFadeOut, int opacidad_begin, int opacidad_end);
	void addMovimiento_FadeIn_FadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal, int tiempoFadeIn, int tiempoFadeOut, int fadein_opacidad_begin, int fadein_opacidad_end, int fadeout_opacidad_begin, int fadeout_opacidad_end);
	void addFadeIn (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int desaparecerAlFinal,int tiempoFadeIn, int opacidad_begin, int opacidad_end);
	void addFadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int desaparecerAlFinal,int tiempoFadeOut, int opacidad_begin, int opacidad_end);
	void addFadeIn_FadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int desaparecerAlFinal, int tiempoFadeIn, int tiempoFadeOut, int fadein_opacidad_begin, int fadein_opacidad_end, int fadeout_opacidad_begin, int fadeout_opacidad_end);
	void addCortinillaIn_Vertical (char nombre[15],int tiempoCortinillaIn, bool UpDown);
	void addCortinillaIn_Horizontal (char nombre[15], int tiempoCortinillaIn, bool LeftRight);
	void addCortinillaOut_Vertical (char nombre[15],int tiempoCortinillaOut, bool UpDown);
	void addCortinillaOut_Horizontal (char nombre[15], int tiempoCortinillaOut, bool LeftRight);
	void addZoomInicial (char nombre[15], double factor);
	void addZoomIn (char nombre[15],int tiempozoom, double factor_begin, double factor_end);
	void addZoomOut (char nombre[15],int tiempozoom, double factor_begin, double factor_end);
	void addRotateInicial (char nombre[15], int grados);
	void addRotateIn (char nombre[15],int tiemporotar, int angle_begin, int angle_end, int factor);
	void addRotateOut (char nombre[15],int tiemporotar, int angle_begin, int angle_end, int factor);
	void addAlineacion (char nombre[15],int align, int valign);
	void addColoresFondoEnAlpha (char nombre[15],int r, int g, int b);
	void addDesaparicion (char nombre[15], int ciclo);
	void addControlAnimation(char nombre[15], ControlAnimation ca);
	// control de música
	void crearSonido (char nombre[15], char archivo[127], int volumen);
	void addEsquema (char nombre[15], int cicloIni, int cicloFin, int tiempoFadeIn, int tiempoFadeOut, int loops);
	void activarMusica();
	void mover();
	void dibujar();
	bool DrawAnimation();
	void escribirTexto (SDL_Surface *sc, int align, int valign, int size, char txt[63], int fgR, int fgG, int fgB, int bgR, int bgG, int bgB, int x, int y);
	// fondos
	void comprobarFondo ();
	void addFondo (int ciclo, int r, int g, int b, char imagen[127], int conAlpha, int posicionFondo);
	// trabajo con foreground
	void addForeground (int cicloIni, int cicloFin, int r, int g, int b, char imagen[127], int conAlpha, int posicionFondo, bool fondoTransparente);
	void addForeground_FadeIn(int tiempoFade);
	void addForeground_FadeOut(int tiempoFade);
	void addForeground_ZoomIn(int tiempozoom, double factor_begin, double factor_end);
	void addForeground_ZoomOut(int tiempozoom, double factor_begin, double factor_end);
	void addForeground_CurtainVIn(int tiempoCurtain);
	void addForeground_CurtainHIn(int tiempoCurtain);
	void addForeground_CurtainVOut(int tiempoCurtain);
	void addForeground_CurtainHOut(int tiempoCurtain);
	// trabajo con variables
	void inVariable (string nombre, string valor);
	string outVariable (string nombre);
	char* convertirRuta (string cadena);
	void imprimirVariables();
	// otros
	bool hacerStop ();
	void unLoad ();
};

#endif /* SDL_ANIMATION_H_ */
