/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */
#ifndef ELEMENTOS_H_
#define ELEMENTOS_H_

#define TRUE 1
#define FALSE 0

#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>
#include <math.h>
#include "defines.h"
#include "fuente.h"
#include "juego.h"
#include "explosion.h"
#include "disparosenemigos.h"
#include "disparojugador.h"
#include "sonido.h"
#include "joystick.h"
#include "unrarlib.h"

class Juego;
class Explosion;
class Audio;
class DisparosEnemigos;
class Joystick;
class BarraVidaEnemigo;
class Fuente;
class DisparoJugador;

class Frame {
	
public:
	SDL_Surface *img;
	SDL_RWops* RWFromRar(char *fileRar, char *file);
	void load(char *path);
	void load(char *path, SDL_Surface *sc);
	void loadFromRAR (char *fileRar, char *path);
	void loadFromRAR (char *fileRar, char *path, SDL_Surface *sc);
	void loadSurface (SDL_Surface *surface);
	void loadSurface (SDL_Surface *surface, SDL_Surface *sc);
	void unLoad();

};

class Elemento {
	
protected:
	float posX, posY, oldX, oldY;
	float dx, dy; // dirección x e y en el caso que sea necesario
	float polarCoseno, polarSeno, polarDistancia; // direccion con coordenadas polares
	float posXNueva, posYNueva; // direccion por posicion
	int tipoCoordenadas; // 0 -> usa cordenadas normales 1-> usa coordenadas polares
	int actualFrame;
	int anteriorFrame;
	int nFrames;
	int cont;
	int activo; // comprueba si el elemento está activo en la pantalla, caso de las balas por ejemplo
					// que se crean con arrays
	int fantasma;
	int impactosParaMorir;
	int actualImpactos;
	Frame *disparo;
	int alpha; // factor alpha aplicado al elemento
	Uint32 *transparente; // valor transparente de la imagen
	int tipoElemento;
	int rebote; // para la funcion selSiguienteFrame_Rebote
	int align, valign;
	int ciclosEntreFrames;
	int tipoRotacion;
	int ciclo;
	int parpadeo, estadoParpadeo; // variables de control de parpadeo
	int cortinilla, maxCortinilla;

public:
	Frame *sprite;
	Elemento ();
	Elemento (int nf);
	Elemento (int nf, int ni);
	void unLoad ();
	void addFrame (int pos, Frame frame);
	void addFrame (Frame frame);
	void selFrame (int nf);
	int getFrames () { return nFrames; }
	void selSiguienteFrame();
	void selSiguienteFrame_Rebote();
	void selSiguienteFrame_ReboteHasta(int hasta);
	Uint32 getTransparente () { return transparente[actualFrame]; }
	void setCiclosEntreFrames (int c) { ciclosEntreFrames = c; }
	int getCiclosEntreFrames() { return ciclosEntreFrames; }
	void setTipoRotacion (int r) { tipoRotacion = r; }
	int getTipoRotacion() { return tipoRotacion; }
	void setParpadeo (int p) { parpadeo = p; }
	int getParpadeo() { return parpadeo; }
	void setCortinilla (int c) { maxCortinilla = c; cortinilla = c; }
	int getCortinilla() { return cortinilla; }
	// límites del elemento en la pantalla
	// se utilizan para saber cuando un elemento tiene que ser descargado al salir de la pantalla
	int getLimiteSup () { return (-getAlto() * 2) -5; }
	int getLimiteInf () { return 480 + (getAlto() * 2) -5; }
	int getLimiteDer () { return 520 + getAncho() * 2 -5; }
	int getLimiteIzq () { return -getAncho() * 2 -5; }
	bool fueraLimitesDePantalla ();
	float getCentroX () { return posX + getAncho()/2; }
	float getCentroY () { return posY + getAlto()/2; }
	float getCentrarXen (float x) { return x - getAncho()/2; }
	float getCentrarYen (float y) { return y - getAlto()/2; }
	void setTipoCoordenadas (int c) { tipoCoordenadas = c; }
	int getTipoCoordenadas () { return tipoCoordenadas; }
	void setDPolarCoseno (float c) { polarCoseno = c; }
	void setDPolarSeno (float s) { polarSeno = s; }
	void setDPolarDistancia (float d) { polarDistancia = d; }
	float getDPolarCoseno () { return polarCoseno; }
	float getDPolarSeno () { return polarSeno; }
	float getDPolarDistancia () { return polarDistancia; }
	void setPorPosicion (int x, int y) { posXNueva = x; posYNueva = y; tipoCoordenadas = PORPOSICION; }
	void setPorPosicionX (int x) { posXNueva = x; posYNueva = posY; tipoCoordenadas = PORPOSICION; }
	void setPorPosicionY (int y) { posXNueva = posX; posYNueva = y; tipoCoordenadas = PORPOSICION; }
	void setDX (float x) { dx = x; }
	void setDY (float y) { dy = y; }
	float getDX () { return dx; }
	float getDY () { return dy; }
	void setAlign (int a) { align = a; }
	void setVAlign (int a) { valign = a; }
	int getAlign () { return align; }
	int getVAlign () { return valign; }
	void addDX () { posX += dx; }
	void addDY () { posY += dy; }
	void setX (float x) { oldX = posX; posX = x; }
	void setY (float y) { oldY = posY; posY = y; }
	void addX (float x) { oldX = posX; posX += x; }
	void addY (float y) { oldY = posY; posY += y; }
	float getOldX () { return oldX; }
	float getOldY () { return oldY; }
	float getX () { return posX; }
	float getY () { return posY; }
	void setTipoElemento (int te) { tipoElemento= te; }
	int getTipoElemento () { return tipoElemento; }
	void mover ();
	int getAncho () { return sprite[actualFrame].img->w; }
	int getAlto () { return sprite[actualFrame].img->h; }
	void setActivo (int a) { activo = a; }
	int getActivo () { return activo; }
	void setImpactosParaMorir (int i) { impactosParaMorir = i; }
	int getImpactosParaMorir () { return impactosParaMorir; }
	int addImpactosParaMorir () { return 1; };
	int getActualFrame () { return actualFrame; }
	void setFantasma (int nf) { fantasma = nf; };
	void conmutaFantasma ();
	int getFantasma () { return fantasma; };
	void setAlpha (int a) { alpha = a; };
	void removeAlpha () { alpha = -1; };
	void dibujar(SDL_Surface *superficie);
	Uint32 SDL_GetPixel(SDL_Surface *surface, int x, int y);
	int SDL_Collide( SDL_Surface *SA , int Ax1 , int Ay1 , Uint32 t1, SDL_Surface *SB , int Bx1 , int By1, Uint32 t2);
	int colision (Elemento *elem);
	SDL_Surface* getSurface() { return sprite[actualFrame].img;};
};

class Jugador : public Elemento {
private:
	int estado; // 0 -> muriendo, 1 -> naciendo, 2 -> vivo, 3 -> activar nacer
	Elemento *sombra, *barrera, *clon_izq, *clon_der, *flare;
	DisparoJugador *disp_jugador;
	bool mostrar, mostrarPoder, mostrarFlare, invisible;
	int ciclosSegundoMovimiento; // cuando se pone a 0 pasa al segundo movimiento cuando va a la derecha o a la izquierda
	bool swEmpezarMovimientoLado; // switch para empezar los ciclos del movimiento de lado derecho e izquierdo
	int tipoMovimiento; // si va para la derecha, izquierda, quieto o estableciendose
	bool actualizarDibujo;
	Uint8 *keys;
	Sint16 joyx, joyy;
	int disparosContinuos; 
	int esperaDisparosContinuos;
	int pulsadoPoder, pulsadoSelPoder;
	int velocidad;
	int teclaArriba, teclaAbajo, teclaDerecha, teclaIzquierda;
	int teclaDisparo_P1, teclaDisparo_P2, teclaUsarPoder_P1, teclaUsarPoder_P2;
	int teclaPoder1, teclaPoder2, teclaPoder3, teclaPoder4;

	bool desplDerecha, desplIzquierda;

public:
	Jugador (int nf);
	DisparoJugador* getDisparoJugador () { return disp_jugador; }
	bool comprobarTeclas(Juego *juego, Joystick *joystick, Audio *sonido, bool compMovimiento, bool compDisparo, bool compPoderes);
	void crearDisparo(Juego *juego, Audio *sonido);
	void moverDerecha() { if (getX() < ( 520 - getAncho() ) ) addX (velocidad); desplDerecha = true; }
	void moverIzquierda() { if (getX()  > 0) addX (-velocidad); desplIzquierda = true; }
	void moverArriba() { if (getY() > 0) addY (-velocidad); }
	void moverAbajo() { if (getY()  <  ( 480 - getAlto() ) ) addY (velocidad); }
	void setMostrarFlare (int f) { mostrarFlare = f; };
	void cargarSprites(SDL_Surface *screen);
	int getEstado () { return estado; };
	void setEstado (int est);
	void morir (Audio *son);
	void moverJugador (Juego *juego);
	void dibujar (SDL_Surface *superficie, Juego *juego, Joystick *joystick, Audio *son);
	void unLoad();
};

class Balon : public Elemento {
private:
	int ciclos; // número de ciclos recorridos desde que se creó el enemigo
	int velHorizontal, velVertical;
public:
	Balon ();
	Balon (int nf);
	void cambiarVelHorizontal() { velHorizontal = -velHorizontal; }
	void mover ();
	void dibujar (SDL_Surface *superficie);
	void iniciar();
};

class Enemigo : public Elemento {
	
protected:
	int ciclos; // número de ciclos recorridos desde que se creó el enemigo
	int numMovimientos; // numero de movimientos del enemigo
	int movimientoActual; // numero de movimiento actual del enemigo. -1 no tiene ningún movimiento
	int movimientos[NUM_MOVIMIENTOS][5];  // array que contiene el tipo de movimiento y ciclo que dura
	int numDisparos; // numero de disparos del enemigo
	int disparoActual; // numero de disparo actual del enemigo. -1 no tiene ningún disparo
	int disparos[NUM_DISPAROS][3];  // array que contiene el tipo de disparo y ciclo que dura
	int tipoRegalo; // cuando muere deja como regalo un agregadores
public:
	Enemigo ();
	Enemigo (int nf);
	void resetearCiclos () { ciclos=0; }
	int getTipoRegalo () { return tipoRegalo; };
	int getCiclos () { return ciclos; };
	void setTipoRegalo (int tr) { tipoRegalo = tr; };
	int getNumMovimientos () { return numMovimientos; };
	int getMovimiento_Tipo ( int nm ) { return movimientos[nm][0]; };
	int getMovimiento_Ciclos ( int nm ) { return movimientos[nm][1]; };
	int getMovimiento_X ( int nm ) { return movimientos[nm][2]; };
	int getMovimiento_Y ( int nm ) { return movimientos[nm][3]; };
	int getMovimiento_nFrame ( int nm ) { return movimientos[nm][4]; };
	int getTipo_Disparo ( ) { return disparos[disparoActual][1]; };
	int getMov_Disparo ( ) { return disparos[disparoActual][2]; };
	// addDisparo -> cic: ciclo en el que empieza, nDisparos: número de disparos, intervalo: ciclos entre los disparos
	void addDisparo(int cic, int tipoDisparo, int movDisparo, int nDisparos, int intervalo);
	bool comprobarDisparo();
};

class GeneralNivel1 : public Enemigo {
	
private:
	int tipo; // 1 -> gaviota marron, 2 -> gaviota blanca, 3 -> nave pequeña

protected:
	int direccionMovimiento (int x, int y); // comprueba la direccion del movimiento según los valores x e y
	int direccionCircular (int mov1, int mov2); // devuelve 0 si el movimiento es de 1 a 2 e 1 al contrario

public:
	GeneralNivel1 ();
	GeneralNivel1 (int nf);
	void duplicarEnFila (GeneralNivel1* est, int espera, int addX, int addY);
	// Se puede añadir diferentes movimientos a un mismo enemigo así:
	// "c" será el número de ciclos que durará el movimiento, siendo -1 hasta q desaparezca
	// de la pantalla
	// "dx" direccion en el ejeX
	// "dy" direccion en el ejeY
	// "tipoMov" será el tipo de movimiento que hará el enemigo puediendo ser:
	// 0 - Movimiento en relación a dx y dy
	// 1 - Movimiento kamikaze
	// 2 - Parado
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	int getTipo() { return tipo; };
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	void mover(int jugadorX);
	void iniciar();
};

class GeneralNivel2 : public Enemigo {
	
private:
	int tipo;
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
	int posMovimiento; // valor que nos dices hacia donde se posiciona la navegrande
								// 0 -> hacia abajo
								// 1 -> hacia la izquierda
								// 2 -> hacia arriba
								// 3 -> hacia la derecha
public:
	GeneralNivel2 ();
	GeneralNivel2 (int nf);
	// Se puede añadir diferentes movimientos a un mismo enemigo así:
	// "c" será el número de ciclos que durará el movimiento, siendo -1 hasta q desaparezca
	// de la pantalla
	// "dx" direccion en el ejeX
	// "dy" direccion en el ejeY
	// "tipoMov" será el tipo de movimiento que hará el enemigo puediendo ser:
	// 0 - Movimiento en relación a dx y dy
	// 1 - Movimiento kamikaze
	// 2 - Parado
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	int getTipo() { return tipo; };
	void setImpacto(int im) { impacto = im; };
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	void mover(int jugadorX);
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class GeneralNivel3 : public Enemigo {
	
private:
	int tipo;
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
public:
	Elemento *sombra;
	Elemento *fuego0, *fuego1;
	GeneralNivel3 ();
	GeneralNivel3 (int nf);
	// Se puede añadir diferentes movimientos a un mismo enemigo así:
	// "c" será el número de ciclos que durará el movimiento, siendo -1 hasta q desaparezca
	// de la pantalla
	// "dx" direccion en el ejeX
	// "dy" direccion en el ejeY
	// "tipoMov" será el tipo de movimiento que hará el enemigo puediendo ser:
	// 0 - Movimiento en relación a dx y dy
	// 1 - Movimiento kamikaze
	// 2 - Parado
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	int getTipo() { return tipo; };
	void setImpacto(int im) { impacto = im; };
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	void mover(int jugadorX);
	void iniciar();
	void dibujar (SDL_Surface *superficie);
	void unLoad();
};

class Marinos : public Enemigo {
	
private:
	int tipo;
	int moverY;
	int tiempoEsconderse; // tiempo que tarda el elemento en meterse bajo agua
						  // en el caso de ser -1 no se meterá nunca
	
public:
	Marinos ();
	Marinos (int nf);
	//void setImpacto(int im) { impacto = im; };
	void setTiempoEsconderse(int t) { tiempoEsconderse = t; };
	void setMoverY(int m) { moverY = m; };
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void mover();
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};


class CocheSimple : public Enemigo {
	
private:
	int tipo;
	int maxCoches; // numero de coches que tiene la clase diferentes
	int cocheActual;
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
	int posMovimiento; // valor que nos dices hacia donde sva el coche
								// 0 -> hacia abajo
								// 1 -> hacia arriba

public:
	CocheSimple ();
	CocheSimple (int nf);
	void setImpacto(int im) { impacto = im; };
	void selCoche(int tipoCoche);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	void mover(int jugadorX);
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class Concha : public Enemigo {
	
private:
	int tipo;
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
	bool escondido;

public:
	Concha ();
	Concha (int nf);
	void setImpacto(int im) { impacto = im; };
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	void mover();
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class Carroza : public Enemigo {
	
private:
	int tipo;
	int maxCarrozas; // numero de carrozas que tiene la clase diferentes
	int carrozaActual;
	int movimientoFrame; // 1 - adelante, -1 - atras
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
	Elemento *tractor;

public:
	Elemento *fuego0, *fuego1, *humo;
	Carroza ();
	Carroza (int nf);
	void setImpacto(int im) { impacto = im; };
	void selCarroza(int tipoCarroza);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void mover(int jugadorX);
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class Enemigo2Frames : public GeneralNivel1 {
	
private:
	int tipo;
	int intervaloFrames;
public:
	Enemigo2Frames ();
	Enemigo2Frames (int nf);
	void setIntervaloFrames(int i) { intervaloFrames = i; }
	void dibujar (SDL_Surface *superficie);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
};

class AutoMarino : public Enemigo2Frames {
	
public:
	AutoMarino ();
	AutoMarino (int nf);
};


class Helicoptero : public Enemigo2Frames {
	
public:
	Helicoptero ();
	Helicoptero (int nf);
};

class CochePolicia : public Enemigo2Frames {
	
public:
	CochePolicia ();
	CochePolicia (int nf);
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void iniciar ();
};

class Taxi : public GeneralNivel1 {
	
public:
	Taxi ();
	Taxi (int nf);
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void iniciar ();
};

class EnemigoConTorreta : public GeneralNivel1 {
	
private:
	int tipo;
	int posTorreta;
public:
	EnemigoConTorreta ();
	EnemigoConTorreta (int nf);
	void addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame);
	int getPosTorreta() { return posTorreta; }
	void dibujar (SDL_Surface *superficie, Jugador *j);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void iniciar ();
};

class Tanque : public EnemigoConTorreta {
	
public:
	Tanque ();
	Tanque (int nf);
};

class Yate : public EnemigoConTorreta {
	
public:
	Yate ();
	Yate (int nf);
};

class Construcciones : public Enemigo {
	
private:
	int tipo;
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave

public:
	Construcciones ();
	Construcciones (int nf);
	int getTipo() { return tipo; };
	void setImpacto(int im) { impacto = im; };
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void mover();
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};


class Mina : public Enemigo {
	
private:
	int tipo;
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
	int dirX;
	
public:
	Mina ();
	Mina (int nf);
	void setImpacto(int im) { impacto = im; };
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void direccion(int d) { if (d==DIR_LEFT) { dirX = -2; } else { dirX = 2; } };
	void mover();
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class Agregador : public Elemento {
	
private:
	int ciclos;
	int tipo; // 0 -> Punto de poder, 1-> Punto de Vida, 2-> Punto de Disparo
	int dx, dy;
public:
	Agregador ();
	Agregador (int nf);
	void setAgX (int x) { setX(x); };
	void setAgY(int y) { setY(y); };
	void setAgDX (int x) { dx = x; };
	void setAgDY(int y) { dy = y; };
	void setAgCiclos(int c) { ciclos = c; };
	void posicionAleatoria();
	void setTipo (int t) { actualFrame = t*9; tipo = t; }
	int getTipo () { return tipo; }
	void dibujar (SDL_Surface *superficie);
	void mover ();
	void iniciar ();
};



class JefeFase1 {
	
private:
	int ciclos;
	int activo;
	int impacto; // se pone a 1 cuando se ha producido un impacto
	int impactosParaMorir;
	Elemento *cuerpo, *brazo_iz, *brazo_de, *pata_iz, *pata_de, *ojos, *sombra;
	int posicionX, posicionY;
	int cerrarOjos;
	int direccionX;
	int direccionX_aux;
	int conMovimiento;
	int conMovimiento_brazo;
	int llamada; // toca las castañuelas
	int finLlamada; // termina tocar castañuelas. Para que salgan elementos
	int sw_primerallamada;
	BarraVidaEnemigo *barra;
	int tiempoBloqueoDisparosEnemigos;

public:
	JefeFase1 (SDL_Surface *screen);
	void bloquearDisparos() { tiempoBloqueoDisparosEnemigos = TIEMPOBLOQUEODISPAROSENEMIGOS; }
	void posicionInicial();
	void dibujar (SDL_Surface *superficie);
	void addX (int x);
	void mover ();
	void setImpacto(int im) { impacto = im; }
	void setActivo(int a) { activo = a; }
	int getActivo() { return activo; }
	int getFinLLamada() { return finLlamada; }
	int colision (Elemento *elem);
	int colisionConJugador(Jugador *j);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	int getImpactosParaMorir() { return impactosParaMorir; }
	void crearDisparo(DisparosEnemigos *disp_ene, Audio *son);
	void iniciar();
	void unLoad();
};

class JefeFase3 : public Enemigo {
	
private:
	int dios; // baco, momo o hercules
	int impacto; // se pone a 1 cuando se ha producido un impacto en la nave
	int ataque1Desde, ataque1Hasta;
	int ataque2Desde, ataque2Hasta;
	int ataque3Desde, ataque3Hasta;
	int fiestaDesde, fiestaHasta;
	int frameExplotado;
	bool enAtaque;
	int ataqueActual;
	int tilesMovimientoPrincipal[4];
	int actualMovimientoPrincipal;
	int ciclosRealizarAtaque;
	BarraVidaEnemigo *barra;
	bool fiesta; // nos dice si los jefes están de fiestassso
	int tiempoFiesta;
	int numFramesMovimientoFiesta;
	int tipoMov; // -1 hacia atras , 1 hacia adelante
	int tiempoBloqueoDisparosEnemigos;
	
public:
	Elemento *fuego0, *humo;
	JefeFase3 ();
	JefeFase3 (int nf);
	void bloquearDisparos() { tiempoBloqueoDisparosEnemigos = TIEMPOBLOQUEODISPAROSENEMIGOS; }
	void crearDisparo(DisparosEnemigos *disp_ene);
	void setDios(int d) { dios = d; };
	int getDios() { return dios; };
	void setFiesta(bool f) { fiesta = f; actualFrame = fiestaDesde; };
	bool getFiesta() { return fiesta; };
	void setFrameExplotado(int f) { frameExplotado = f; };
	int getFrameExplotado() { return frameExplotado; };
	void setNumFramesMovimientoFiesta(int n) { numFramesMovimientoFiesta = n; };
	int getNumFramesMovimientoFiesta() { return numFramesMovimientoFiesta; };
	void setImpacto(int im) { impacto = im; };
	void setCiclosRealizarAtaque(int c) { ciclosRealizarAtaque = c; };
	void setMovimientoPrincipal(int pos, int n) { tilesMovimientoPrincipal[pos] = n; };
	void setAtaque1(int desde, int hasta) { ataque1Desde = desde; ataque1Hasta = hasta; };
	void setAtaque2(int desde, int hasta) { ataque2Desde = desde; ataque2Hasta = hasta; };
	void setAtaque3(int desde, int hasta) { ataque3Desde = desde; ataque3Hasta = hasta; };
	bool getEnAtaque() { return enAtaque; };
	void setMovimientoFiesta(int desde, int hasta) { fiestaDesde = desde; fiestaHasta = hasta; };
	void crearBarraVida (SDL_Surface *screen, char texto[32], int tv, int posX, int posY);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};


class JefeFase2 {
	
private:
	int ciclos;
	int activo;
	int impacto; // se pone a 1 cuando se ha producido un impacto
	int impactosParaMorir;
	Elemento *cabeza, *patas;
	Elemento *tent1, *tent2, *tent3, *tent4;
	Elemento *disparo1, *disparo2;
	Elemento *alquitran1[MAX_ALQUITRAN], *alquitran2[MAX_ALQUITRAN];
	Elemento *mancha;
	bool tentaculoFuera;
	int nTentaculoFuera;
	int posicionX, posicionY;
	BarraVidaEnemigo *barra;
	int tiempoBloqueoDisparosEnemigos;

	bool ataque, empezarAtaque, pararAlquitran;
	int ciclosAtaque;

public:
	JefeFase2 (SDL_Surface *screen);
	void bloquearDisparos() { tiempoBloqueoDisparosEnemigos = TIEMPOBLOQUEODISPAROSENEMIGOS; }
	void sacarTentaculo();
	void posicionInicial();
	void dibujar (SDL_Surface *superficie);
	void mover ();
	void setImpacto(int im) { impacto = im; }
	void setActivo(int a) { activo = a; }
	int getActivo() { return activo; }
	int colision (Elemento *elem);
	int colisionConJugador(Jugador *j);
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	int getImpactosParaMorir() { return impactosParaMorir; }
	void crearDisparo(DisparosEnemigos *disp_ene, Audio *son);
	void iniciar();
	void iniciarAlquitran();
	void unLoad();
};

class TorreBalneario : public Enemigo {

private:
	int impacto; // se pone a 1 cuando se ha producido un impacto
	int abrir, cerrar;
	bool torreAbierta;
	int tiempoTorreAbierta;
	int movimiento;
	
public:
	TorreBalneario ();
	TorreBalneario (int nf);
	void abrirTorre() { abrir = 1; torreAbierta = 1; movimiento = 1; }
	void cerrarTorre() { cerrar = 1; abrir = 0; torreAbierta = 0; 	numDisparos = 0; disparoActual = -1; selFrame(3); }
	int addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp);
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class BioPic : public Enemigo {

private:
	int mostrar;
	bool biopicFuera;
	int tiempoBioPicFuera;
	int tiempoEsperaEntrada;
	int tiempoBloqueoDisparosEnemigos;
	
public:
	BioPic ();
	BioPic (int nf);
	void bloquearDisparos() { tiempoBloqueoDisparosEnemigos = TIEMPOBLOQUEODISPAROSENEMIGOS; }
	bool getBiopicFuera() { return biopicFuera; }
	void crearDisparo(DisparosEnemigos *disp_ene);
	void mostrarBioPic() { activo = 1; mostrar = 1; biopicFuera = 1; }
	void ocultarBioPic() { activo = 0; mostrar = 0; numDisparos = 0; disparoActual = -1; }
	void iniciar();
	void dibujar (SDL_Surface *superficie);
};

class ElCano : public Elemento {
private:
	int ciclos; // número de ciclos recorridos desde que se creó el enemigo
	int velocidad;
	Elemento *rampa;
	bool rampaAbierta;

public:
	ElCano ();
	void cargarSprites();
	void setRampaAbierta(bool r) { rampaAbierta = r; }
	bool getRampaAbierta() { return rampaAbierta; }
	void setVelocidad(int v) { velocidad = v; }
	void mover ();
	void dibujarRampa (SDL_Surface *superficie);
	void dibujar (SDL_Surface *superficie);
	void iniciar();
};

class BarraDeCarga : public Elemento {
private:
	int estadoCarga, totalCarga, ancho, alto;
	SDL_Rect dest, src;
	Elemento *borde, *barra, *barra_fin;

public:
	BarraDeCarga (SDL_Surface *screen, int tc, int posX, int posY);
	void setTotalCarga (int tc) { estadoCarga = 1; totalCarga = tc; }
	int getTotalCarga () { return totalCarga; }
	void actualizarCarga() { estadoCarga++; }
	void dibujar (SDL_Surface *superficie);
	void setPosicion(int posX, int posY);
};


class BarraVidaEnemigo {
private:
	int estadoVida, totalVida;
	SDL_Rect rOr, rDest;
	Elemento *limite, *txt;
	SDL_Surface *surface_vida;

public:
	BarraVidaEnemigo (SDL_Surface *screen, char texto[32], int tv, int posX, int posY);
	void setTotalVida (int tv) { estadoVida = tv; totalVida = tv; }
	int getTotalVida () { return totalVida; }
	void actualizarVida(int ev) { estadoVida = ev; }
	void dibujar (SDL_Surface *superficie);
	void setPosicion(int posX, int posY);
	SDL_Surface* crearTextoConSombra(SDL_Surface *screen, char texto[127]) ;
};

class MapaAgua {

private:
	int indice, indice_in;
	Elemento *suelo[12];
	int maxmap;

	char *mapa, *nt;

	int velocidad;
	int numTilesActual;
public:
	MapaAgua();
	char* BufferFromRar(char *fileRar, char *file);
	void dibujarMapa (SDL_Surface *screen);
	void Cargar(SDL_Surface *screen);
	void iniciar();
	void stop() { velocidad = 0; }
	void play() { velocidad = 2; }
	void setVelocidad(int v) { velocidad = v; };
	int getVelocidad() { return velocidad; };
	void addVelocidad() { velocidad++; };
	void removeVelocidad() { velocidad--; };
	void unLoad();
};

#endif /* ELEMENTOS_H_ */
