/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */
 
 #include "mapa.h"
 
/*****************************
**
** CLASE MAPA
**
******************************/

 Mapa::Mapa() {
	indice = MAXMAP - 169;
	indice_in = 0;

	numTilesActual = 0;
	 
	int i;
	for ( i=0; i<MAXTILES; i++ ) {
		suelo[i]= new Elemento();
	}
	
	char rar[128];
	Frame frPalmera, frFarola;
	
	sprintf (rar, "%s/miscelanea.tim", DATA_DIR);
	frPalmera.loadFromRAR(rar, "palmeras.timp");
	
	for ( i=0; i<MAXPALMERAS; i++ ) {
		palmeras[i]= new Elemento();
		palmeras[i]->addFrame(frPalmera);
		palmeras[i]->setX(192);
		palmeras[i]->setVAlign(VALIGN_BOTTOM);
	}
	
	frFarola.loadFromRAR(rar, "farola.timp");
	
	for ( i=0; i<MAXFAROLA; i++ ) {
		farola[i]= new Elemento();
		farola[i]->addFrame(frFarola);
		farola[i]->setX(359);
		farola[i]->setVAlign(VALIGN_BOTTOM);
	}
	
	fondofinal = new Elemento();
	
	velocidad = 2;

 }
 
void Mapa::dibujarTilesMapa (SDL_Surface *screen) {
	int aux = velocidad;
	velocidad = 0;
	dibujarMapa(screen);
	velocidad = aux;
}
 
void Mapa::dibujarMapa (SDL_Surface *screen) {
	int i,j,x,y,t;
	int dim = 40;
	int tiles_ancho = 13;
	int tiles_alto = 13;

	indice_in+=velocidad;
	if (indice_in >= dim) {
		indice_in = 0;
		indice -= tiles_ancho;
	}
	
	if (indice <= 0) {
		indice = MAXMAP-169;
		indice_in = 0;
	}
	int cont=0;
	for (i=0; i<tiles_alto; i++) {
		for (j=0; j<tiles_ancho; j++) {
			t = mapa[indice+(i*tiles_ancho+j)];
			cont++;
			x = j * dim;
			y = (i-1) * dim + indice_in;
			suelo[t]->selFrame(0);
			suelo[t]->setX(x);
			suelo[t]->setY(y);
			suelo[t]->dibujar(screen);
			
		}		
	}

}

int Mapa::elementoLibre (int ele) {
	int i;
	switch (ele) {
		case PALMERA:
			for (i=0; i < MAXPALMERAS; i++) {
				if (palmeras[i]->getActivo() == 0) {
					return i;
				}
			}			
		break;
		case FAROLA:
			for (i=0; i < MAXFAROLA; i++) {
				if (farola[i]->getActivo() == 0) {
					return i;
				}
			}			
		break;
	}	
	return	-1;
}

void Mapa::dibujarElementos (SDL_Surface *screen, Juego *j) {
	int numLibre, i;
	switch (j->getNivel()) {
		case 2:
			
			for ( i=0; i<MAXPALMERAS; i++ ) {
				if (palmeras[i]->getActivo()) {
					palmeras[i]->addY(velocidad);
					palmeras[i]->dibujar(screen);
					if (palmeras[i]->fueraLimitesDePantalla()) {
						palmeras[i]->setActivo(0);
					}
				}
			}
			if ( (j->getCiclos()%60==0) && (velocidad>0) ) {
				numLibre = elementoLibre(PALMERA);
				if (numLibre!=-1) {
					palmeras[numLibre]->setActivo(1);
					palmeras[numLibre]->setY(-84);
				}
			}
			
			for ( i=0; i<MAXFAROLA; i++ ) {
				if (farola[i]->getActivo()) {
					farola[i]->addY(velocidad);
					farola[i]->dibujar(screen);
					if (farola[i]->fueraLimitesDePantalla()) {
						farola[i]->setActivo(0);
					}
				}
			}
			if ( (j->getCiclos()%120==0) && (velocidad>0) ) {
				numLibre = elementoLibre(FAROLA);
				if (numLibre!=-1) {
					farola[numLibre]->setActivo(1);
					farola[numLibre]->setY(-84);
				}
			}
		
		break;		
	}
		
	
}

char* Mapa::BufferFromRar(char *fileRar, char *file)
{
  char *data_ptr;
  unsigned long data_size;

  if(urarlib_get(&data_ptr, &data_size, file, fileRar, "password")) 
  {
	return data_ptr;
    //if(data_ptr != NULL) free(data_ptr);
  } else 
  { 
    printf("Error al descomprimir!\n"); 
  }
  return data_ptr;
}

void Mapa::CargarNivel (int l, SDL_Surface *screen) {
	Frame frSuelo[MAXTILES], frFondoFinal;
	char tmpfile[50];
	char rarMapa[128];
	int i;
	// cargar mapa
	
	sprintf (rarMapa, "%s/mapa%d.tma", DATA_DIR,  l);
	
	mapa = BufferFromRar(rarMapa, "mapa.map");
	
	// cargar tiles del suelo
	
	nt = BufferFromRar(rarMapa, "ntiles.dsc");
	numTilesActual = atoi (nt);
	
	for (i=0; i<=numTilesActual; i++) {
		sprintf (tmpfile, "tiles\\tile%d.timb", i);
		frSuelo[i].loadFromRAR(rarMapa, tmpfile);
		suelo[i]->addFrame(0, frSuelo[i]);
	}

	if (l!=2) {
		sprintf (tmpfile, "%s/final%d.tim", DATA_DIR, l);
		frFondoFinal.loadFromRAR(tmpfile, "fondo_final_fase.timp");
		fondofinal->sprite[0].img = frFondoFinal.img;
		fondofinal->setX(0);
		fondofinal->setY(0);
	}
	
}


void Mapa::dibujarFinal (SDL_Surface *screen, Juego *j) {
	if (j->getNivel()!=2) {
		fondofinal->dibujar(screen);
	}
}

void Mapa::iniciar (int i) {
	// empezar el mapa desde el principio
	
	int su = 169 * (i+1);
	if ( su > MAXMAP ) {
		su = su -MAXMAP;
	}
	indice = MAXMAP - su;  //indice = MAXMAP - 169;
	indice_in = 0;
	
	for ( i=0; i<MAXFAROLA; i++ ) {
		farola[i]->setActivo(0);
	}
	
	for ( i=0; i<MAXPALMERAS; i++ ) {
		palmeras[i]->setActivo(0);
	}

	setVelocidad(VEL_MAPA);
	
}

void Mapa::unLoad () {
	for (int i=0; i<=numTilesActual; i++) {
		suelo[i]->unLoad();
	}
	fondofinal->unLoad();
}
